# OBJ - timestamped JSON objects
#
# obj/rss.py
#
# Copyright 2017,2018 B.H.J Thate
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice don't have to be included.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# 14-07-2018 As the creator of this file, I disclaim all rights on  this file. 
#
# Bart Thate
# Heerhugowaard
# The Netherlands

""" rss module (needs feedparser). """

__version__ = 13

## imports

from obj.base import Obj
from obj.bus import Bus
from obj.clock import Repeater
from obj.store import Store
from obj.rss.url import file_time, get_feed, strip_html
from obj.task import launch

import datetime
import html
import http
import logging
import os
import random
import re
import time
import urllib
import urllib.error
import xml

gotparser = False

try:
    import feedparser
    gotparser = True
except ImportError:
    pass

## defines

class Seen(Obj):

    pass

class Cfg(Obj):

    pass

seen = Seen()
seen.urls = []

defaults = Cfg()
defaults.display_list = ["title", "summary", "link"]
defaults.kind = "rss"
defaults.summary = []

## instances

bus = Bus()
store = Store()

## exceptions

class ENODATE(Exception):

    """
        no date could be determined.

    """
    
## classes

class Feed(Obj):

    pass

class Rss(Obj):

    pass


class Fetcher(Obj):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.cfg = defaults

    def display(self, obj):
        result = ""
        for key in self.cfg.display_list:
            if key == "summary" and self.cfg.summary:
                skip = True
                for k in self.cfg.summary:
                    if k in obj.link:
                       skip = False
                       break
                if skip:
                    continue
            data = obj.get(key, None)
            if data:
                data = data.replace("\n", "")
                result += "%s - " % strip_html(data.rstrip())
        if result:
            return result[:-3].rstrip()

    def fetch(self, url):
        from obj.rss.url import to_time
        nr = 0
        objs = []
        for o in list(get_feed(url))[::-1]:
            if o.link in seen.urls:
                continue
            seen.urls.append(o.link)
            nr += 1
            feed = Feed(o)
            objs.append(feed)
            if "updated" in feed:
                try:
                    date = file_time(to_time(feed.updated))
                    feed.save(stime=date)
                    continue
                except ENODATE as ex:
                    logging.info("ENODATE %s" % str(ex))
            elif "published" in feed:
                try:
                    date = file_time(to_time(feed.published))
                    feed.save(stime=date)
                    continue
                except ENODATE as ex:
                    logging.info("ENODATE %s" % str(ex))
            feed.save()
        logging.warn("got %s feeds" % len(objs))
        seen.save()
        for o in objs:
            txt = self.display(o)
            bus.announce(txt)
        return nr

    def run(self):
        res = []
        thrs = []
        nr = len(seen.urls)
        for fn, obj in store.find("obj.rss.Rss"):
            if "rss" not in obj:
                continue
            if not obj.rss.strip():
                continue
            launch(self.fetch, obj.rss)

    def start(self, *args, **kwargs):
        global seen
        seen.last()
        self.cfg.last()
        repeater = Repeater(600, self.run)
        repeater.start()

    def stop(self):
        global seen
        seen.save()

## commands

def fetcher(event):
    """ run a fetch batch. """
    rss = Fetcher()
    res = rss.run()
    event.reply("fetched %s" % ",".join([str(x) for x in res]))

def rss(event):
    """ add a shopitem to the shopping list. """
    if not event.rest:
        event.reply("rss <url>")
        return
    o = Rss()
    o.rss = event.rest
    o.save()
    event.reply("ok 1")

def version(event):
    event.reply("OBJ.RSS %s" % __version__)
