# OB - Framework to program bots
#
# ob/utils/rss.py
#
# Copyright 2017,2018 B.H.J Thate
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice don't have to be included.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# 13-07-2018 As the creator of this file, I disclaim all rights on  this file. 
#
# Bart Thate
# Heerhugowaard
# The Netherlands

""" rss module (needs feedparser). """

__version__ = 1

## imports

from obj.base import Obj
from obj.utils import get_exception

import datetime
import logging
import os
import random
import re
import time
import urllib

gotparser = False

try:
    import feedparser
    gotparser = True
except ImportError:
    pass

## defines

# 17 Jul 2018 00:00:00 +0200       

timestrings = [
               "%a, %d %b %Y %H:%M:%S %z",
               "%d %b %Y %H:%M:%S %z",
               "%d %b %Y %H:%M:%S",
               "%a, %d %b %Y %H:%M:%S",
               "%d %b %a %H:%M:%S %Y %Z",
               "%d %b %a %H:%M:%S %Y %z",
               "%a %d %b %H:%M:%S %Y %z",
               "%a %b %d %H:%M:%S %Y",
               "%d %b %Y %H:%M:%S",
               "%a %b %d %H:%M:%S %Y",
               "%Y-%m-%d %H:%M:%S",
               "%Y-%m-%dt%H:%M:%S+00:00",
               "%a, %d %b %Y %H:%M:%S +0000",
               "%d %b %Y %H:%M:%S +0000",
              ]

## helpers

def file_time(timestamp):
    return str(datetime.datetime.fromtimestamp(timestamp)).replace(" ", os.sep) + "." + str(random.randint(111111, 999999))

def get_feed(url):
    if not gotparser:
        logging.warn("feedparser is missing.")
        return []
    result = []
    if not url or not "http" in url:
        logging.debug("%s is not an url." % url)
        return result
    try:
        result = feedparser.parse(get_url(url).data)
    except urllib.error.HTTPError as ex:
        logging.error("%s %s" % (url, ex.msg))
        return result
    if "entries" in result:
        for entry in result["entries"]:
            yield Obj(entry)

def get_url(*args, **kwargs):
    url = urllib.parse.urlunparse(urllib.parse.urlparse(args[0]))
    req = urllib.request.Request(url, headers={"User-Agent": useragent()})
    resp = urllib.request.urlopen(req)
    resp.data = resp.read()
    logging.debug("%s %s %s" % (resp.status, resp.reason, url))
    return resp

def strip_html(text):
    clean = re.compile('<.*?>')
    return re.sub(clean, '', text)

def to_time(daystr):
    from obj.rss import ENODATE
    daystr = daystr.strip()
    if "," in daystr:
        daystr = " ".join(daystr.split(None)[1:7])
    if "(" in daystr:
        daystr = " ".join(daystr.split(None)[:-1])
    res = 0
    for tstring in timestrings:
        try:
            res = time.mktime(time.strptime(daystr, tstring))
        except:
            try:
                res = time.mktime(time.strptime(" ".join(daystr.split()[:-1]), tstring))
            except:
                pass
        if res:
           break
    if not res:
        raise ENODATE(daystr)
    return res

def useragent(): 
    return 'Mozilla/5.0 (X11; Linux x86_64) OB #%s +https://bitbucket.org/bthate/ob)' % __version__

def unescape(text):
    return html.parser.HTMLParser().unescape(text)
